<?php
/*
Plugin Name: EbaySalesLister
Plugin URI: http://blog.airness.de/2007/05/01/wordpress-plugin-ebay-sales-lister/
Description: The Ebay Sales Lister is a plugin that lets you display a list of ebay sales in the sidebar of your WordPress blog.
Version: 0.3
Author: Sebastian Spier
Author URI: http://blog.airness.de/
*/

/*  Copyright 2007 Sebastian Spier (email : blog@airness.de)

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

@define("EBAYSALESLISTER_VERSION", "0.3");
@define("EBAYSALESLISTER_DIRPATH",'/wp-content/plugins' . strrchr(dirname(__FILE__),'/') . '/');
@define("EBAYSALESLISTER_DEBUG", false);

class EbaySalesLister {
	var $ebayUrl = "http://cgi6.ebay.de/ws/eBayISAPI.dll?ViewListedItems&rdir=0&rows=#ROWS#&userid=#USERID#";
	var $ebayWorldUrl = "http://myworld.ebay.de";
	// array for storing the created sales objects
	var $sales = array();
	var $langStrings = array();
	
	// options from wordpress db table "wp_options";
	var $options;
	
	// option representatives used within this script (so option names within $options could be changed)
	var $ebayUsername;
	var $itemCount;
	var $title;
	var $ebayWebsite;
	var $timeFormat;
	var $language;
	
	// for debugging
	var $seconds;
	var $debug = array();
	
	// constructur
	function EbaySalesLister() {
		// get options from DB
		$options = get_option('ebaySalesLister');
		// init language strings
		$this->initLangStrings();
		// init options in local instance
		$this->setOptions($options);
	}	

	function debug($msg=false) {
		if($msg){
			$this->debug[] = (time()- $this->seconds) . 'secs > ' . $msg;
		}
		else {
			foreach ($this->debug as $line) {
				$content.=$line . "\n"; 
			}
			return $content;
		}
	}

	function initLangStrings() {
		// german
		$this->langStrings[de][Tage] = Tage;
		$this->langStrings[de][Tag] = Tag;
		$this->langStrings[de][Stunden] = Stunden;			
		$this->langStrings[de][Stunde] = Stunde;
		$this->langStrings[de][Minuten] = Minuten;
		$this->langStrings[de][Minute] = Minute;
		$this->langStrings[de][Sekunden] = Sekunden;
		$this->langStrings[de][Sekunde] = Sekunde;
		$this->langStrings[de][Endet_am] = Endet;
		$this->langStrings[de][Endet_in] = Endet;
		$this->langStrings[de][Preis] = Preis;
		$this->langStrings[de][keineArtikel] = "Momentan keine Angebote von";
		
		// english
		$this->langStrings[en][Tage] = days;
		$this->langStrings[en][Tag] = day;
		$this->langStrings[en][Stunden] = hours;			
		$this->langStrings[en][Stunde] = hour;		
		$this->langStrings[en][Minuten] = minutes;
		$this->langStrings[en][Minute] = minute;
		$this->langStrings[en][Sekunden] = seconds;
		$this->langStrings[en][Sekunde] = second;	
		$this->langStrings[en][Endet_am] = Ends;
		$this->langStrings[en][Endet_in] = Ends;
		$this->langStrings[en][Preis] = Price;
		$this->langStrings[en][keineArtikel] = "Currently no sales for";
		
		// french
		$this->langStrings[fr][Tage] = jours;
		$this->langStrings[fr][Tag] = jour;
		$this->langStrings[fr][Stunden] = heures;			
		$this->langStrings[fr][Stunde] = heure;		
		$this->langStrings[fr][Minuten] = minutes;
		$this->langStrings[fr][Minute] = minute;
		$this->langStrings[fr][Sekunden] = secondes;
		$this->langStrings[fr][Sekunde] = seconde;	
		$this->langStrings[fr][Endet_am] = Fin;
		$this->langStrings[fr][Endet_in] = Fin;
		$this->langStrings[fr][Preis] = Prix;
		$this->langStrings[fr][keineArtikel] = 'Pas darticle disponnible en ce moment';	

		// turkey
		$this->langStrings[tr][Tage] = 'gn';
		$this->langStrings[tr][Tag] = 'gn';
		//$this->langStrings[tr][Stunden] = saatler;			
		$this->langStrings[tr][Stunden] = saat;			
		$this->langStrings[tr][Stunde] = saat;		
		//$this->langStrings[tr][Minuten] = dakikalar;
		$this->langStrings[tr][Minuten] = dakika; // dak.
		$this->langStrings[tr][Minute] = dakika;
		//$this->langStrings[tr][Sekunden] = saniyeler;
		$this->langStrings[tr][Sekunden] = saniye; // sn.
		$this->langStrings[tr][Sekunde] = saniye;	
		$this->langStrings[tr][Endet_am] = son;
		$this->langStrings[tr][Endet_in] = 'kalan sre';
		$this->langStrings[tr][Preis] = fiyat;
		$this->langStrings[tr][keineArtikel] = "Suan satis yok";	

		//
        $this->langStrings[sv][Tage] = dagar;
        $this->langStrings[sv][Tag] = dag;
        $this->langStrings[sv][Stunden] = timmar;           
        $this->langStrings[sv][Stunde] = timme;       
        $this->langStrings[sv][Minuten] = minuter;
        $this->langStrings[sv][Minute] = minut;
        $this->langStrings[sv][Sekunden] = sekunder;
        $this->langStrings[sv][Sekunde] = sekund;   
        $this->langStrings[sv][Endet_am] = Avslutas;
        $this->langStrings[sv][Endet_in] = 'Avslutas om';
        $this->langStrings[sv][Preis] = Pris;
        $this->langStrings[sv][keineArtikel] = 'Inga aktiva auktioner fr'; 
	
	}
	
	/* write options from $options[] to local representatives */
	function setOptions($options){
		$this->options = $options;
		$this->ebayUsername = $options['ebayUsername'];
		$this->ebayWebsite = $options['ebayWebsite'];
		$this->itemCount = $options['itemCount'];
		$this->title = $options['title'];
		$this->timeFormat = $options['timeFormat'];
		$this->language = $options['language'];
		
		$this->langStrings = $this->langStrings[$this->language];
		
		if ($this->itemCount == "all") 
			$this->itemCount = 1000;
	}
	
	// extract site content from ebay
	function fetch_url2($url) {
		$fp = fopen($url,"r");
		$c = 0;
		if ($fp) {
			while ( (!feof($fp)) && ($c<2000) ) {
		    	$line = fgets($fp,1024);
		      	$site_content .= $line;
		      	$c++;
		    }
		}
		return $site_content;
	}
	
	/* Fetch remote url, using cURL if available or fallback to file_get_contents */
	function fetch_url($url){
		
		/* Use cURL if it is available, otherwise attempt fopen */
		if(function_exists('curl_init') && function_exists('curl_exec_redirect') ){ 
			$this->debug('Using (Modified) cURL for request...');
						
			$ch = curl_init();
	
			@curl_setopt($ch, CURLOPT_HEADER, true);
			@curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
			@curl_setopt($ch, CURLOPT_FAILONERROR, true);	//Die if we received >404
			@curl_setopt($ch, CURLOPT_CONNECTTIMEOUT, 5);	//x seconds to give up this connect
					
			curl_setopt($ch, CURLOPT_URL, $url);

			// grab URL and pass it to the browser
			$data = $this->curl_exec_redirect($ch);
			
			//$this->debug('Result: ' . $data);			

			if ((curl_errno($ch)) || ($data===true)) {
				$data=false;
			}

 			// close curl resource, and free up system resources
 			curl_close($ch);
			
			$this->debug('cURL closed.');			
		} 
		/* If cURL is not installed use file_get_contents */
		else {
			$this->debug('cURL not installed. Using file_get_contents()...');
			$data=@file_get_contents ( $url );
			$this->debug('Result: ' . $data);	
		} 
			
 		return $data;
	}	
	
	// inject parameters to get value
	function createUrl() {
		$url = $this->ebayUrl;
		$url = str_replace("#ROWS#",$this->itemCount,$url);
		$url = str_replace("#USERID#",$this->ebayUsername,$url);
		//$url = str_replace("#EBAYWEBSITE#",$this->ebayWebsite,$url);
		//echo $url;
		return $url;
	}

	// parse ebay sales table
	function getSales() {
		$url = $this->createUrl();
		$this->debug("URL: " . $url);
		
		$content = $this->fetch_url($url);
		
		$tableMatcher = '|<table border="1" cellpadding="3" bordercolor="#FFFFFF" cellspacing="0">(.*?)</table>|sm';
		preg_match($tableMatcher,$content,$matches);
		$content = $matches[1];
		//if ($this->debug) echo $content;
		
		$trMatcher = '/<tr bgcolor="#(?:ffffff|efefef)">.+?<\/tr>/sm';
		preg_match_all($trMatcher,$content,$matches);
			
		$lines = $matches[0];
		$this->debug("LINES" . $lines);
		
		//print_r($this->debug);
		
		$saleMatcher = '|href="(http://.*?=\d*)".*?(\d{2}\.\d{2}\.\d{2} \d{2}:\d{2}:\d{2}).*?>(.*?\d+,\d+).*?<td color="">(.*?)</td>|sm';
		
		foreach($lines as $line) {	
			// only add sale if regexp matches
			if (preg_match($saleMatcher,$line,$s)) {
				$link = str_replace("ebay.de",$this->ebayWebsite,$s[1]);
				$newSale = new ESL_Sale($link,$s[2],strip_tags($s[3]),strip_tags($s[4]));
				// add item if time could be parsed
				if ($newSale->endTimeTimestamp != "") {
					$this->sales[] = $newSale;
				}
			}
		}
		
		$this->debug("SALES" . $this->sales);
	}
	
	// output function for blog
	function displaySales() {	
		$out = "";
		$out .= "<h2 class='ebay'>" . $this->title . "</h2>";
		$out .= "<ul>";
		if (count($this->sales) != 0) {
			$count = 0;
			foreach($this->sales as $sale) {
				if ($count >= $this->itemCount) {
					break;
				}
				$out .= "<li>";
				$out .= "<a href='$sale->link'>$sale->title</a><br>";
				if ($this->timeFormat == "end time")
					$out .= $this->langStrings[Endet_am] .": ";
				else
					$out .= $this->langStrings[Endet_in] .": ";
				$out .= $sale->showDate($this->timeFormat,$this->langStrings);
				$out .= "<br>";
				$out .= $this->langStrings[Preis] . ": $sale->price";
				$out .= "</li>";
				$count++;
			}
		}
		else {
			$out .= "<li>" . $this->langStrings[keineArtikel] . " <a href='$this->ebayWorldUrl/$this->ebayUsername'>$this->ebayUsername</a></li>";
		}
		$out .= "</ul>";
		
		echo $out;
	}
	
	// register listeners
	function init() {
		// Do Widget-specific code
		if (function_exists('register_sidebar_widget') ) {   
			// This registers our widget so it appears with the other available
			// widgets and can be dragged and dropped into any active sidebars.
			register_sidebar_widget('Ebay Sales Lister', array(&$this,'widget'));

			// This registers our optional widget control form. Because of this
			// our widget will have a button that reveals a 300x100 pixel form.
			register_widget_control('Ebay Sales Lister', array(&$this,'control_form'), 300, 420);
		} 
		// add normal options page (found at Settigns -> "Ebay Sales Lister")
		add_action('admin_menu', array(&$this,'add_pages'));	
	}
	
	
	function widget() {
		//$options = get_option('ebaySalesLister');
		$ebaySales = new EbaySalesLister();
		$ebaySales->getSales();
		$ebaySales->displaySales();
	}
	
	/*
	function widget($args) {
		print_r($args);
		if ($args == null) {
			$args = array(
				before_widget => "<li>",
				before_title => "<h2 class='ebay'>",
				after_title => "</h2><ul>",
				after_widget => "</ul></li>"
			);
		}	
		// $args is an array of strings that help widgets to conform to
		// the active theme: before_widget, before_title, after_widget,
		// and after_title are the array keys. Default tags: li and h2.
		extract($args);

		// Each widget can store its own options. We keep strings here.
		$options = get_option('ebaySalesLister');
		$title = $options['title'];

		// These lines generate our output. Widgets can be very complex
		// but as you can see here, they can also be very, very simple.
		echo $before_widget . $before_title . $title . $after_title;
		//$this->display(); //main call to get imonline icon
		//$ebaySales = new EbaySalesLister();
		$this->getSales();
		$this->display();
		echo $after_widget;
	}	
	*/
	
	// show control form
	function control_form() {	
		// get our options
		$options = get_option('ebaySalesLister');
				
		// set defaults if no $options exist
		if ( !is_array($options) ) {
			$options = array(
				'title'=>'Ebay Sales',
				'ebayUsername'=>'',
				'ebayWebsite'=>'ebay.com',
				'itemCount'=>'all',
				'timeFormat'=>'ends',
				'language'=>'en'
				//'affiliCode'=>''
			);
		}
		
		// form was submitted
		if ( $_POST['esl_submit'] ) {
			// get form data
			$options['title'] = strip_tags(stripslashes($_POST['esl_title']));
			$options['ebayUsername'] = strip_tags(stripslashes($_POST['esl_ebayUsername']));
			$options['ebayWebsite'] = strip_tags(stripslashes($_POST['esl_ebayWebsite']));
			$options['itemCount'] = $_POST['esl_itemCount'];
			$options['timeFormat'] = $_POST['esl_timeFormat'];
			$options['language'] = $_POST['esl_language'];
			//$options['affiliCode'] = $_POST['esl_affiliCode'];
					
			// save options
			update_option('ebaySalesLister', $options);
						
			//header('Location: '.get_bloginfo('wpurl').'/wp-admin/options-general.php?page=ebaySalesLister&updated=true')
			//die();			
		}
		
		?>
		<div style="">
			<table border="0" cellspacing="0" cellpadding="0">
                <tr>
					<td width="100px" style="font-weight:bold;text-align:right;">
						<label>Title:</label>
					</td>
					<td>
						<input style="width: 170px;" name="esl_title" type="text"
						value="<?php echo $options['title']?>" />
					</td>
				</tr>
                <tr>
					<td style="font-weight:bold;text-align:right;">
						<label>Ebay Username</label>
					</td>
					<td>
						<input style="width: 170px;" name="esl_ebayUsername" type="text"
						value="<?php echo $options['ebayUsername']?>" />
					</td>
				</tr>	
                <tr>
					<td style="font-weight:bold;text-align:right;">
						<label>Ebay Website</label>
					</td>
					<td>
						<select name="esl_ebayWebsite">
						<?php
							$ebaySite = array(
								"ebay.de",
								"ebay.com",
								"ebay.fr",
								"ebay.co.uk");
							
							foreach($ebaySite as $site) {
								echo '<option value="' . $site . '"';
								if ($options['ebayWebsite'] == $site) echo ' selected="selected"';
								echo '>' . $site . '</option>';
							}
						?>
						</select>	
					</td>
				</tr>					
				<tr>
					<td style="font-weight:bold;text-align:right;"> 
						<label>Articles</label>
					</td>
					<td>
						<select name="esl_itemCount">
						<?php
							foreach(array(all,10,5,3) as $count) {
								echo '<option value="' . $count . '"';
								if ($options['itemCount'] == $count) echo ' selected="selected"';
								echo '>' . $count . '</option>';
							}
						?>						
						</select>
					</td>
				</tr>	
                <tr>
					<td style="font-weight:bold;text-align:right;">
						<label>Time Format</label>
					</td>
					<td>
						<select name="esl_timeFormat">
						<?php
							foreach(array("end time","time remaining") as $value) {
								echo '<option value="' . $value . '"';
								if ($options['timeFormat'] == $value) echo ' selected="selected"';
								echo '>' . $value . '</option>';
							}
						?>	
						</select>
					</td>
				</tr>
                <tr>
					<td style="font-weight:bold;text-align:right;">
						<label>Language</label>
					</td>
					<td>
						<select name="esl_language">
						<?php
							$languages = array(de,en,fr,tr,sv);
							
							foreach($languages as $lang) {
								echo '<option value="' . $lang . '"';
								if ($options['language'] == $lang) echo ' selected="selected"';
								echo '>' . $lang . '</option>';
							}
						?>
						</select>
					</td>
				</tr>						
			
<!--				
                <tr>
					<td style="font-weight:bold;text-align:right;">
						<label>ebay affili code</label>
					</td>
					<td>
						<input style="width: 200px;" name="esl_affiliCode" type="text" 
						value="<?php echo $options['affiliCode']?>"/>
					</td>
				</tr>	
-->				
			</table>
			<input type="hidden" name="esl_submit" value="1" />
		</div>
		<?php
	}
	
	// show widget control
	function widget_control() {
		$this->control_form();
	}	
	
	// show standart control
    function standard_control() {
         ?>
		 <div class="wrap">
         <h2>Ebay Sales Lister Options</h2>
         <div style="margin-top:20px;">
		 <form action="<?php echo get_bloginfo('wpurl') ?>/wp-admin/options-general.php?page=ebaySalesLister&updated=true" method="post">
       <!--  <form action="" method="post"> -->
          <?php

               $this->control_form();

         ?>
         <p class="submit"><input type="submit" value="Save changes &raquo;"></p>
         </form></div></div>
		 <?php
        }	
	
	/* add normal admin options page */
	function add_pages() {
		// http://codex.wordpress.org/Adding_Administration_Menus
		// add_options_page(page_title, menu_title, access_level/capability, file, [function]);
		add_options_page("Ebay Sales Lister Options", "Ebay Sales Lister", 10, "ebaySalesLister", array(&$this,'standard_control'));
	}
		
}

/* Class representing one single ebay sale */
class ESL_Sale {
	var $title;
	var $endTime;
	var $endTimeTimestamp;
	var $price;
	var $link;
	
	// constructor
	function ESL_Sale($link, $endTime, $price, $title) {
		$this->title = $title;
		$this->endTime = $endTime;
		$this->endTimeTimestamp = $this->createTimestamp($endTime);
		$this->price = $price;
		$this->link = $link;
	}
	
	function showDate($timeFormat,$langStrings) {
		if ($timeFormat == "end time") {
			return $this->endTime;
		}
		else if ($timeFormat == "time remaining") {
			return $this->timeRemaining($langStrings);
		}
		else {
			return "Invalid format.";
		}
	}
	
	function createTimestamp($ebayDate) {
		if (preg_match("/(\d{2})\.(\d{2})\.(\d{2}) (\d{2}):(\d{2}):(\d{2})/",$ebayDate,$m)) {
			$convertedDate = "20$m[3]-$m[2]-$m[1] $m[4]";
			$timestamp = mktime($m[4], $m[5], $m[6], $m[2], $m[1], $m[3]);
			return $timestamp;
		}
		else {
			return "";
		}
	}
	
	function timeRemaining($langStrings) {
		$now = date('Y-m-d H:i:s', time());
		$endTime = date('Y-m-d H:i:s', $this->endTimeTimestamp);
		$diff = $this->get_time_difference($now,$endTime);
		
		//print_r($langStrings);
						
		$remaining = "";
		if ($diff[days] > 0) {
			$remaining .= "$diff[days] " . (($diff[days] == 1) ? $langStrings[Tag] : $langStrings[Tage]);
			$remaining .= " $diff[hours] " . (($diff[hours] == 1) ? $langStrings[Stunde] : $langStrings[Stunden]);
		}
		else if ($diff[hours] > 0){
			$remaining .= "$diff[hours] " . (($diff[hours] == 1) ? $langStrings[Stunde] : $langStrings[Stunden]);
			$remaining .= " $diff[minutes] " . (($diff[minutes] == 1) ? $langStrings[Minute] : $langStrings[Minuten]);
		}	
		else if ($diff[minutes] > 0){
			$remaining .= "$diff[minutes] " . (($diff[minutes] == 1) ? $langStrings[Minute] : $langStrings[Minuten]);
			$remaining .= " $diff[seconds] " . (($diff[seconds] == 1) ? $langStrings[Sekunde] : $langStrings[Sekunden]);
		}	
		else if ($diff[seconds] > 0){
			$remaining .= "$diff[seconds] " . (($diff[seconds] == 1) ? $langStrings[Sekunde] : $langStrings[Sekunden]);
		}	
		else if(diff == false) {
			$remaining .= "Time calculation error";
		}
		else {
			$remaining .= "Auction ended";
		}	

		return $remaining;
	}
	
	/**
	 * Function to calculate date or time difference.	 
	 *
	 * Function to calculate date or time difference. Returns an array or
	 * false on error.
	 *
	 * @author       J de Silva                             <giddomains@gmail.com>
	 * @copyright    Copyright &copy; 2005, J de Silva
	 * @link         http://www.gidnetwork.com/b-16.html    Get the date / time difference with PHP
	 * @param        string                                 $start
	 * @param        string                                 $end
	 * @return       array
	 */
	function get_time_difference( $start, $end ) {
	    $uts['start']      =    strtotime( $start );
	    $uts['end']        =    strtotime( $end );
	    if( $uts['start']!==-1 && $uts['end']!==-1 )
	    {
	        if( $uts['end'] >= $uts['start'] )
	        {
	            $diff    =    $uts['end'] - $uts['start'];
	            if( $days=intval((floor($diff/86400))) )
	                $diff = $diff % 86400;
	            if( $hours=intval((floor($diff/3600))) )
	                $diff = $diff % 3600;
	            if( $minutes=intval((floor($diff/60))) )
	                $diff = $diff % 60;
	            $diff    =    intval( $diff );            
	            return( array('days'=>$days, 'hours'=>$hours, 'minutes'=>$minutes, 'seconds'=>$diff) );
	        }
			/*
	        else
	        {
	            trigger_error( "Ending date/time is earlier than the start date/time", E_USER_WARNING );
	        }
			*/
	    }
		/*
	    else
	    {
	        trigger_error( "Invalid date/time data detected", E_USER_WARNING );
	    }
		*/
	    return( false );
	}	
	
}

function listEbaySales() {
	$ebaySales = new EbaySalesLister();
	$ebaySales->widget();
}

$ebaySales = new EbaySalesLister();

// Run our code later in case this loads prior to any required plugins.
add_action('plugins_loaded', array(&$ebaySales,'init'));

?>